!*  <<< SUBMTXOP >>> --------------------------------------------------
!======================================================================

!*AFFINE   -- Affine transformation.
!+
SUBROUTINE AFFINE(G,WORK,iext,jext,A,irc)
  real(4)  :: G(iext,jext), WORK(iext,jext)
  real(8)  :: A(6)

! AFFINE:
!         PERFORM AFFINE TRANSFORMATION.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           A      : COEFFICIENTS OF TRANSFORMATION
!         INPUT,OUTPUT/
!           G      : MATRIX
!         OUTPUT/
!           irc    : return CODE ( 0 : NORMAL END )
!         WORK AREA/
!           WORK   : (ORIGINAL INPUT IMAGE IS KEPT)
!
! Other subroutine called:
!         (P) AFFIN1
!
! Note:
!         THE ORIGINAL INPUT IMAGE IS KEPT IN WORK ON return.
!
!                    WORK(I,J) ----> G(U,V)
!
!        (  U  )    ( A(1)   A(2) ) ( I )   ( A(5) )
!        (     ) =  (             ) (   ) + (      )
!        (  V  )    ( A(3)   A(4) ) ( J )   ( A(6) )
!
!--
!     AUTHOR: S.ICHIKAWA  1985-12-19
!     UPDATE: S.ICHIKAWA  1986-02-24   (KEEP ORIGINAL DATA)
!     UPDATE: M. Hamabe   1996-11-13   (For auto-doc)
! ---------------------------------------------------------------------

  real(8)  :: ADET, AI, AJ, AX, AY
  
  ADET=A(1)*A(4)-A(2)*A(3)
  if ( ABS(ADET)<=1.0D-8 ) then
     irc=1000
     return
  end if
  
  do J=1,jext
     AJ=dble(J)
     do I=1,iext
        AI=dble(I)
        AX=((AI-A(5))*  A(4) +(AJ-A(6))*(-A(2)))/ADET
        AY=((AI-A(5))*(-A(3))+(AJ-A(6))*  A(1) )/ADET
        if ( AX<1.0D0 .or. AX>dble(iext) .or.  & 
             & AY<1.0D0 .or. AY>dble(jext) ) then
           VAL=0.0
        else
           II=INT(AX)
           JJ=INT(AY)
           call AFFIN1(AX,AY,G(II,JJ),G(II+1,JJ),G(II,JJ+1),G(II+1,JJ+1),VAL)
        end if
        WORK(I,J)=VAL
     end do
  end do
  
  do J=1,jext
     do I=1,iext
        W        =G(I,J)
        G(I,J)   =WORK(I,J)
        WORK(I,J)=W
     end do
  end do
  irc=0
  
  return

END SUBROUTINE AFFINE

!---------------------------------------------------------------------
SUBROUTINE AFFIN1(AX,AY,G1,G2,G3,G4,VAL)

!     *** LINEAR INTERPOLATION ***

  IMPLICIT real*8 (A-B)
  
  A1=AX-INT(AX)
  A2=AY-INT(AY)
  B1=(G2-G1)*A1+G1
  B2=(G4-G3)*A2+G3
  VAL=(B2-B1)*A2+B1
  
  return

END SUBROUTINE AFFIN1

!======================================================================

!*REPLAC   -- replace matrix ( (90*n)deg rot & reverse ).
!+
SUBROUTINE REPLAC(G1,iext1,jext1,G2,iext2,jext2,MODE,irc)
  DIMENSION  G1(iext1,jext1),G2(iext2,jext2)

! REPLAC:
!         REPLACE MATRIX.
!
! Arguments:
!         INPUT/
!           G1     : MATRIX
!           iext1  : I-DIMENSION OF THE MATRIX G1
!           jext1  : J-DIMENSION OF THE MATRIX G1
!           iext2  : I-DIMENSION OF THE MATRIX G2
!           jext2  : J-DIMENSION OF THE MATRIX G2
!           MODE   : REPLACE MODE
!                      1 :   0 DEG. ROT.
!                      2 :  90 DEG. ROT.
!                      3 : 180 DEG. ROT.
!                      4 : 270 DEG. ROT.
!                      5 : REVERSE(I-DER) &   0 DEG. ROT.
!                      6 : REVERSE(I-DER) &  90 DEG. ROT.
!                      7 : REVERSE(I-DER) & 180 DEG. ROT.
!                      8 : REVERSE(I-DER) & 270 DEG. ROT.
!         OUTPUT/
!           G2     : MATRIX
!           irc    : return CODE ( 0 : NORMAL END )
!
!--
!     AUTHOR: S.ICHIKAWA  1986-01-15
!     UPDATE:
! ---------------------------------------------------------------------
!
!     *** REJECT UNMATCHED-SIZE ***
  if ( MOD(MODE,2)==1 ) then
     if ( iext1/=iext2 .or. jext1/=jext2 ) then
        irc=1000
        return
     end if
  else
     if ( iext1/=jext2 .or. jext1/=iext2 ) then
        irc=1000
        return
     end if
  end if

!     *** REPLACE ***
  if ( MODE==1 ) then
     G2(1:iext2,1:jext2)=G1(1:iext2,1:jext2)
  else if ( MODE==2 ) then
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(iext1-J+1,I)
        end do
     end do
  else if ( MODE==3 ) then
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(iext1-I+1,jext1-J+1)
        end do
     end do
  else if ( MODE==4 ) then
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(J,jext1-I+1)
        end do
     end do
  else if ( MODE==5 ) then
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(iext1-I+1,J)
        end do
     end do
  else if ( MODE==6 ) then
     G2(1:iext2,1:jext2)=G1(1:jext2,1:iext2)
  else if ( MODE==7 ) then
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(I,jext1-J+1)
        end do
     end do
  else
     do J=1,jext2
        do I=1,iext2
           G2(I,J)=G1(iext1-J+1,jext1-I+1)
        end do
     end do
  end if

  irc=0

  return

END SUBROUTINE REPLAC

!======================================================================

!*REDUCE   -- reduce matrix (use pixel average).
!+
SUBROUTINE REDUCE(G1,iext1,jext1,G2,iext2,jext2,KRDF,irc)
  real :: G1(iext1,jext1),G2(iext2,jext2)

! REDUCE:
!         REDUCE MATRIX USING PIXEL AVERAGE.
!
! Arguments:
!         INPUT/
!           G1     : MATRIX
!           iext1  : I-DIMENSION OF THE MATRIX G1
!           jext1  : J-DIMENSION OF THE MATRIX G1
!           iext2  : I-DIMENSION OF THE MATRIX G2   iext2=iext1/KRDF
!           jext2  : J-DIMENSION OF THE MATRIX G2   jext2=jext1/KRDF
!           KRDF   : REDUCTION FACTOR (POSITIVE INTEGER)
!         OUTPUT/
!           G2     : MATRIX
!           irc    : return CODE ( 0 : NORMAL END )
!
!--
!     AUTHOR: S.ICHIKAWA  1986-01-15
!     UPDATE:
! ---------------------------------------------------------------------

!     *** REJECT UNMATCHED SIZE ***
  if ( iext2*KRDF>iext1 .or. jext2*KRDF>jext1 ) then
     irc=1000
     return
  end if

!     *** INITIALIZE OUTPUT MATRIX ***
  G2(1:iext2,1:jext2)=0.0

!     *** REDUCTION ***
  RDF2=KRDF*KRDF
  do J=1,jext2*KRDF
     JJ=(J-1)/KRDF+1
     do I=1,iext2*KRDF
        II=(I-1)/KRDF+1
        G2(II,JJ)=G2(II,JJ)+G1(I,J)/RDF2
     end do
  end do

  irc=0

  return

END SUBROUTINE REDUCE

!======================================================================

!*REDUCEB   -- reduce matrix (use pixel average & blank).
!+
SUBROUTINE REDUCEB(G1,iext1,jext1,G2,iext2,jext2,KRDF,BLANK,irc)
  real :: G1(iext1,jext1), G2(iext2,jext2)

! REDUCE:
!         REDUCE MATRIX USING PIXEL AVERAGE.
!
! Arguments:
!         INPUT/
!           G1     : MATRIX
!           iext1  : I-DIMENSION OF THE MATRIX G1
!           jext1  : J-DIMENSION OF THE MATRIX G1
!           iext2  : I-DIMENSION OF THE MATRIX G2   iext2=iext1/KRDF
!           jext2  : J-DIMENSION OF THE MATRIX G2   jext2=jext1/KRDF
!           KRDF   : REDUCTION FACTOR (POSITIVE INTEGER)
!         OUTPUT/
!           G2     : MATRIX
!           irc    : return CODE ( 0 : NORMAL END )
!
!--
!     AUTHOR: M. Hamabe  2002-10-12
!     UPDATE:
! ---------------------------------------------------------------------

!     *** REJECT UNMATCHED SIZE ***
   if ( iext2*KRDF>iext1 .or. jext2*KRDF>jext1 ) then
      irc=1000
      return
   end if

!     *** INITIALIZE OUTPUT MATRIX ***
   G2(1:iext2,1:jext2)=0.0

!     *** REDUCTION ***
   
   RDF2=KRDF*KRDF
   
   do j=1, jext2
      do i=1, iext2
         iflg=0
         do jj=1, krdf
            do ii=1, krdf
               dum = g1( (i-1)*krdf+ii, (j-1)*krdf+jj )
               if ( dum==blank ) then
                  iflg=iflg+1
               else 
                  g2(i,j) = g2(i,j) + dum/rdf2
               end if
            end do
         end do
         if ( iflg>=1 ) g2(i,j)=blank
      end do
   end do
   
   irc=0
   
   return

 END SUBROUTINE REDUCEB

!======================================================================

!*EXPAND   -- expand matrix (linear interpolation).
!+
SUBROUTINE EXPAND(G1,iext1,jext1,G2,iext2,jext2,KEXF,irc)
  real :: G1(iext1,jext1), G2(iext2,jext2)

! EXPAND:
!         EXPAND MATRIX USING LINEAR INTERPOLATION.
!
! Arguments:
!         INPUT/
!           G1     : MATRIX
!           iext1  : I-DIMENSION OF THE MATRIX G1
!           jext1  : J-DIMENSION OF THE MATRIX G1
!           iext2  : I-DIMENSION OF THE MATRIX G2
!           jext2  : J-DIMENSION OF THE MATRIX G2
!           KEXF   : EXPANSION FACTOR (POSITIVE INTEGER)
!         OUTPUT/
!           G2     : MATRIX
!           irc    : return CODE ( 0 : NORMAL END )
!
! Other subroutine called:
!        (P) EXPAN1
!
!--
!     AUTHOR: S.ICHIKAWA  1986-01-15
!     UPDATE: S.ICHIKAWA  1987-07-28
! ---------------------------------------------------------------------

!     *** INITIALIZE ***
  G2(1:iext2,1:jext2)=0.0

!     *** SET UNINTERPOLATED VALUE ***
  do J=1,jext2
     JJ=(J-1)/KEXF+1
     if ( JJ<=jext1 ) then
        do I=1,iext2
           II=(I-1)/KEXF+1
           if ( II<=iext1 ) then
              G2(I,J)=G1(II,JJ)
           end if
        end do
     end if
  end do

!     *** SET INTERPOLATED VALUE ***
  do J=1,jext2
     PJ=(real(J)-0.5)/KEXF+0.5
     JJ=PJ
     if ( JJ>=1 .and. JJ+1<=jext1 ) then
        do I=1,iext2
           PI=(real(I)-0.5)/KEXF+0.5
           II=PI
           if ( II>=1 .and. II+1<=iext1 ) then
              call EXPAN1(PI,PJ,G1(II,JJ),G1(II+1,JJ), &
                   &            G1(II,JJ+1),G1(II+1,JJ+1),VAL)
              G2(I,J)=VAL
           end if
        end do
     end if
  end do
  
  irc=0
  
  return

END SUBROUTINE EXPAND

!----------------------------------------------------------------------
SUBROUTINE EXPAN1(X,Y,G1,G2,G3,G4,VAL)

  D1=X-INT(X)
  D2=Y-INT(Y)
  P1=(G2-G1)*D1+G1
  P2=(G4-G3)*D1+G3
  VAL=(P2-P1)*D2+P1

  return

END SUBROUTINE EXPAN1







