!*  <<< SUBSTAT >>> ---------------------------------------------------

!======================================================================

!*STATRJ   -- compute statistics (2-dim; n-times of f-sigma reject's).
!+
SUBROUTINE STATRJ(G,iext,jext,IS,IE,JS,JE,FACT,NCYCLE, &
     &            GMAX,GMIN,GMEAN,GSD,NREJ,irc)
  real :: G(iext,jext)

! STATRJ:
!         COMPUTE STATISTICS (2-DIM; N-TIMES OF F-SIGMA REJECT'S).

! Arguments:
!         INPUT/
!           G      : MATRIX FOR WHICH STATISTICAL VALUES ARE COMPUTED
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           IS     : START-POINT OF I-DIMENSION
!           IE     : END  -POINT OF I-DIMENSION
!           JS     : START-POINT OF J-DIMENSION
!           JE     : END  -POINT OF J-DIMENSION
!           FACT   : FACTOR OF F-SIGMA REJECTION
!           NCYCLE : NUMBER OF CYCLES FOR F-SIGMA REJECTION
!         OUTPUT/
!           GMAX   : MAX. VALUE         OF THE MATRIX G
!           GMIN   : MIN. VALUE         OF THE MATRIX G
!           GMEAN  : MEAN               OF THE MATRIX G
!           GSD    : STANDARD DEVIATION OF THE MATRIX G
!           NREJ   : NUMBER OF REJECTED PIXEL
!           irc    : return CODE
!                      0: NORMAL END
!                    101: IS<1,  102: IE>iext, 111: JS<1, 112: JE>jext
!                    121: IS>IE, 122: JS>JE,
!                    1000: (IE-IS+1)*(JE-JS+1)<1
!
!--
!     AUTHOR: S.ICHIKAWA 1985-10-19
!     UPDATE: S.ICHIKAWA 1985-12-12   CHANGE METHOD OF ADDITION
!     UPDATE: S.ICHIKAWA 1986-04-23   FROM STATCS
!     UPDATE: S.ICHIKAWA 1986-05-21   CORRECT NREJ
!     UPDATE: S.OKAMURA  1986-06-09   REVISED FROM STATRJ SUBROUTINE
!     UPDATE: S.ICHIKAWA 1987-05-10   CORRECTION
!     UPDATE: M.Hamabe   1999-10-14   add 'irc=0'
!     ---------------------------------------------------------------------
      
  real(8) :: S, SS, T, TT, W

  GMAX =0.0
  GMIN =0.0
  GMEAN=0.0
  GSD  =0.0

  irc=101
  if ( IS<1    ) return
  irc=102
  if ( IE>iext ) return
  irc=111
  if ( JS<1    ) return
  irc=112
  if ( JE>jext ) return
  irc=121
  if ( IS>IE   )   return
  irc=122
  if ( JS>JE   )   return
  
  NP=(IE-IS+1)*(JE-JS+1)
  irc=1000
  if ( NP<=1 )    return
  irc=0
  
  !     *** COMPUTE MEAN AND SIGMA IN WHOLE AREA ***
  S   = 0.0D0
  SS  = 0.0D0
  do J=JS, JE
     T  = 0.0D0
     TT = 0.0D0
     do I = IS, IE
        V  =  dble(G(I,J))
        T  =  T + V
        TT = TT + V*V
     end do
     S  = S  + T
     SS = SS + TT
  end do
  GMEAN=S/dble(NP)
  W    =(SS-(S*S)/dble(NP))/dble(NP-1)
  GSD  =sqrt(W)
  
  !     *** N-SIGMA REJECTION  (NCYCLE-TIMES) ***
  do N=1,NCYCLE
     GMAX =-1.0E20
     GMIN = 1.0E+20
     S   = 0.0D0
     SS  = 0.0D0
     NREJ = 0
     do J = JS, JE
        T  = 0.0D0
        TT = 0.0D0
        do I = IS, IE
           if ( abs(G(I,J)-GMEAN)<=GSD*FACT ) then
              T =T +dble(G(I,J))
              TT=TT+dble(G(I,J))**2
              if ( G(I,J)>=GMAX ) GMAX=G(I,J)
              if ( G(I,J)<=GMIN ) GMIN=G(I,J)
           else
              NREJ=NREJ+1
           end if
        end do
        S =S +T
        SS=SS+TT
     end do
     NP0=NP-NREJ
     if ( NP0<=1 ) then
        irc=10000
        return
     end if
     GMEAN = S/dble(NP0)
     W     = (SS-(S*S)/dble(NP0))/dble(NP0-1)
     GSD   = sqrt(W)
  end do
  
  irc=0
  
  return

END SUBROUTINE STATRJ

!======================================================================

!*STCS1    -- compute statistics (max./min./mean/S.D. for 1-dim. data).
!+
SUBROUTINE STCS1 (D,N,DMAX,DMIN,DMEAN,DSD)
  real :: D(N)

! STCS1:
!         COMPUTE STATISTICS (MAX./MIN./MEAN/S.D. FOR 1-DIM. DATA).

! Arguments:
!         INPUT/
!           D      : ARRAY FOR WHICH STATISTICAL VALUES ARE COMPUTED
!           N      : NUMBER OF DATA IN ARRAY D(N)
!         OUTPUT/
!           DMAX   : MAX. VALUE         OF THE ARRAY D
!           DMIN   : MIN. VALUE         OF THE ARRAY D
!           DMEAN  : MEAN               OF THE ARRAY D
!           DSD    : STANDARD DEVIATION OF THE ARRAY D
!
!--
!     AUTHOR: S.OKAMURA  1986-06-19
!     UPDATE:
! ---------------------------------------------------------------------

  real(8) :: S,SS,W
  
  DMAX =-1.0E20
  DMIN =1.0E20
  S   =0.0D0
  SS  =0.0D0
  
  do I=1,N
     if ( D(I)< DMIN ) DMIN=D(I)
     if ( D(I)> DMAX ) DMAX=D(I)
     S =S + D(I)
     SS=SS+D(I)*D(I)
  end do
  DMEAN=S/dble(N)
  W    =(SS-(S*S)/dble(N))/dble(N-1)
  DSD  =sqrt(W)
  
  return

END SUBROUTINE STCS1

!======================================================================

!*STRJ1    -- compute statistics (1-dim; n-times of f-sigma reject's).
!+
SUBROUTINE STRJ1 (D,N,FACT,NCYCLE,DMAX,DMIN,DMEAN,DSD,NREJ,irc)
  real :: D(N)

! STRJ1:
!         COMPUTE STATISTICS (1-DIM; N-TIMES OF F-SIGMA REJECTIONS).
!
! Arguments:
!         INPUT/
!           D      : ARRAY FOR WHICH STATISTICAL VALUES ARE COMPUTED
!           N      : NUMBER OF DATA IN ARRAY D(N)
!           FACT   : FACTOR OF F-SIGMA REJECTION
!           NCYCLE : NUMBER OF CYCLES FOR F-SIGMA REJECTION
!         OUTPUT/
!           DMAX   : MAX. VALUE         OF THE ARRAY D
!           DMIN   : MIN. VALUE         OF THE ARRAY D
!           DMEAN  : MEAN               OF THE ARRAY D
!           DSD    : STANDARD DEVIATION OF THE ARRAY D
!           NREJ   : NUMBER OF REJECTED DATA
!           irc    : return CODE  0=NORMAL END, 1000= NREJ>N-2

!--
!     AUTHOR: S.OKAMURA  1986-06-19
!     UPDATE:
! ---------------------------------------------------------------------
  
  real(8) :: S,SS,W
  
  DMAX =0.0
  DMIN =0.0
  DMEAN=0.0
  DSD  =0.0
  
  !     *** COMPUTE MEAN AND SIGMA IN WHOLE AREA ***
  S   =0.0D0
  SS  =0.0D0
  do I=1,N
     S =S + D(I)
     SS=SS+D(I)*D(I)
  end do
  DMEAN=S/dble(N)
  W    =(SS-(S*S)/dble(N))/dble(N-1)
  DSD  =sqrt(W)
  
  !     *** N-SIGMA REJECTION  (NCYCLE-TIMES) ***
  NREJ=0
  do KK=1,NCYCLE
     DMAX=-1.0E20
     DMIN=1.0E+20
     S   =0.0D0
     SS  =0.0D0
     NREJ0=0
     do I=1,N
        if ( abs(D(I)-DMEAN)<=DSD*FACT ) then
           S =S +dble(D(I))
           SS=SS+dble(D(I))*dble(D(I))
           if ( D(I)>=DMAX ) DMAX=D(I)
           if ( D(I)<=DMIN ) DMIN=D(I)
        else
           NREJ0=NREJ0+1
        end if
     end do
     NP=N-NREJ0
     if ( NP<=1 ) then
        irc=10000
        return
     end if
     DMEAN=S/dble(NP)
     W    =(SS-(S*S)/dble(NP))/dble(NP-1)
     W=abs(W)
     DSD  =sqrt(W)
     NREJ =NREJ+NREJ0
  end do
  
  irc=0
  
  return

END SUBROUTINE STRJ1

