Subroutine SUBPROF(title,gdata,iext,jext,psize,sky,band,NLP,LOUT, &
     &   Xc,Yc,FINC,POSANG,ANGSGN,ANGAX,boa_0,blank, &
     &   NSTEP,MPA,NSINC,NAKA,NITER,NROUGH,L_omit1,L_omit2, &
     &   INTER,CENTER,LOGSC,ERRLOG,NOTFRE,ANGFRE,RATFIX)
  
  !     Find the isophote profile and parameters of an object in an image.
  
  !     This routine is used to find the surface brightness profile of an
  !     object in an image by describing the object as a set of elliptical
  !     contours.  The center of the object is calculated before hand with
  !     the axes centroiding command.  This routine uses this center as
  !     a starting point for the profile calculation.  The profile is found
  !     by sampling the image grid with a set of circles at each integral
  !     step in pixels.  The average value of the pixels along the circle
  !     is the mean surface brightness of the contour.  Low order sine
  !     and cosine transforms are taken along the contour to derive
  !     its center, position angle, and ellipticity.  after these are found
  !     for the entire range in radius specified, the contours are adjusted
  !     to more accurately fit the isophotes.  The first iteration generally
  !     turns the original circles into ellipses with varying position angles
  !     and eccentricities as a function of major axis length.
  
  !     High accuracy sinc interpolation is used to find the values of the
  !     pixels along the inner 15 contours.  Outside of this either a lower
  !     (INNER NSINC CONTOURS)
  !     accuracy, but faster sinc interpolation can be used, or an even
  !     faster and simpler bilinear interpolation.  The contour position
  !     angles and eccentricities are median filtered between iterations
  !     to avoid the effects of random noise on these parameters.
  
  !     Angles of the contours are calculated assuming that position angle
  !     0 is at the top of the image, and that there are no reflections.
  !     This can be over-riden with a keyword.
  
  !     The results of the profile calculation are stored in a common
  !     block for future examination or storage by the plot, print, save
  !     and get commands.  The surface brightness of the profile as a function
  !     of contour major axis is loaded into a spectrum buffer for ease in
  !     examination and manipulation.
  
  ! Keywords:     RADIUS        Maximum semi-major axis length of contour
  !               N             the number of contour steps which are
  !                             calculated in 1-pixel interval.
  !               ITER=(N1,N2)  This controls the number of iterations
  !                             'N1' is the number of iterations using
  !                             simple interpolation. 'N2' is the
  !                             number using sinc interpolation.
  !               NOINT         End the program when iterations given by
  !                             iter are finished.
  !               SCALE=F       The pixel scale in ''/pixel.
  !               CENTER        solve for the contour centers
  !               PA=F          Position angle of the 'top' of the
  !                             image.  Specify f<0 if the image has
  !                             a mirror reflection.
  !               LOG           Produce a log-file of variation of F.c.
  !               DLOGR=F       Make contour steps nearly-log scale
  !                             INTERVAL.
  !                             'F' is the logr interval along the major
  !                             axis.
  !               M=N           'N' is the number of inner contour steps
  !                             which are caluculted using high accuracy
  !                             sinc interpolation routine.
  
  ! author:       Tod R. Lauer    5/5/83         Lick Observatory
  !               This program is a modification of a program written
  !               by Steve Kent at MIT.
  
  !     modified by mark Beduz   DAO
  !     Modified to read the contour center from the VISTA variables AXR and AXC
  !     Rather than from the variables YC and XC so that the user
  !     can set the center without first having to define a box and call axes.
  !     The user may also vary the center if they so desire between executions
  !     of profile by changing the value in AXR and/or AXC.
  !     Modified to call UNRECKEY if an incorrect keyword is spotted.
  !     also accepts abreviated forms of the keywords.                 85/5/16
  !     Modified to include PROFILE.INC to make it consistent with other
  !     routines using these variables.                                86/1/13
  !     Modified to accept redirection to allow the user to produce permanent
  !     listing of the data displayed on the screen.                  86/02/11
  !     Modified to add control-c as a method of aborting a profile.  86/02/26
  !     New keyword NOTFRE added but not documented.   This keyword controls
  !     whether the postion angle is corrected for the inner 3 contours.  It
  !     was added to allow both options to be easlily tested and compared
  !     during debugging.                                             86/03/03
  !     
  !     Modified by Masaru Hamabe  DAO (Kiso Obs.T.A.O.,Japan)
  !     Detailed descriptions of the modifications are given elsewhere.
  !     It should be noted that BIN.FOR (interpolation routine) was also 
  !     modified.
  !     Several mistakes in the original version were also corrected.
  !                                                               86/02/08
  
  !     Modified by Masaru Hamabe 1988-06-28
  !     Modified by Masaru Hamabe 1989-07-23   (ANGFRE)
  !     Modified by Masaru Hamabe 1999-08-16   (increase NPROFI)
  !     Modified by Masaru Hamabe 1999-09-02   (devided from vprof.f)
  !     Modified by Masaru Hamabe 1999-09-02   (introduce L_omit1/2)
  !     Modified by Masaru Hamabe 2002-10-21   (DLINE=blank)
  !     Modified by Masaru Hamabe 2008-06-12   (for Fortran90)
  
  !______________________________________________________________________
  
  parameter (NPROFI=500, MEDANG=7, SKPINT=-1.1E38)
  !                      =  the maximum number of isophotes
  
  character   TITLE*18, band*2, YN
  integer*4   IDR(NPROFI)
  real*4      profs(12,NPROFI)
  !     -           profs( 1,n) : Xc
  !     -           profs( 2,n) : Yc
  !     -           profs( 3,n) : position angle
  !     -           profs( 4,n) : eccentricity
  !     -           profs( 5,n) : mean intensity
  !     -           profs( 6,n) : cos(4E)
  !     -           profs( 7,n) : sin(4E)
  !     -           profs( 8,n) : cos(2E)
  !     -           profs( 9,n) : sin(2E)
  !     -           profs(10,n) : cos(E)
  !     -           profs(11,n) : sin(E)
  !     -           profs(12,n) : derivativ of intensity
  real*4      gdata(iext,jext), SORT(MEDANG), eb(8,3), SUMFC(8)
  real*4      PRF(4000), V(NPROFI), HOLD(NPROFI)
  logical     INTER,ROUGH,CENTER,LOGSC,ERRLOG
  logical     NOTFRE,ANGFRE,RATFIX,OMIT
  data        PI/3.141593/
  
  if ( L_omit1*L_omit2==0 ) then
     OMIT=.FALSE.
  else
     OMIT=.TRUE.
  end if
  
  print '(/'' *****  PROFILE computation  *****''// &
       &   '' Contours centered on ('',F6.2,'','',F7.2, &
       &   '') at position angle ='',F7.2)',  Xc, Yc, ANGAX
  
  if ( ERRLOG ) &
       &   write(NLP ,'(/'' *****  PROFILE computation  *****''// &
       &   '' Contours centered on ('',F6.2,'','',F7.2, &
       &   '') at angle'',F7.2)') Xc, Yc, ANGAX
  
  !     Initialize profile parameters at center using results from the AX block.
  
  profs(1,1) = Xc
  profs(2,1) = Yc
  ANGdum = 90.0-ANGAX
  if ( ANGdum<0.0 ) ANGdum = ANGdum+180.0
  profs(3,1) = ANGdum
  profs(4,1) = 0.0
  X = Xc
  Y = Yc
  ANG = ANGdum
  ECC = 0.0
  if ( RATFIX ) then
     ECC = sqrt( 1.0 - boa_0*boa_0 )
     print *, 'Axial raio is fixed to ',boa_0
  end if
  !     VT
  NSTEP = NSTEP+1
  NAKA  = NAKA +1
  NSINC = NSINC+1
  if ( NSTEP>NPROFI-1 ) NSTEP = NPROFI-1
  dumx1 = iext-Xc
  dumx2 = Xc-1
  dumy1 = jext-Yc
  dumy2 = Yc-1
  dumx  = max(dumx1,dumx2)-1
  dumy  = max(dumy1,dumy2)-1
  dum   = dumx*dumx+dumy*dumy
  Ndum  = sqrt(dum)
  if ( NSTEP>Ndum ) NSTEP = Ndum
  NDERIV = NSTEP
  !VB
  DLINE = blank
  print *,'blank=',blank
  !@@@  DLINE = -100000.0
  !     INITIALIZE INTERPOLATION
  D = BINSET(gdata,jext,iext,DLINE)
  profs(5,1) = XBIN(X,Y,gdata,jext,iext)
  V(1) = profs(5,1)
  if ( NSTEP<=1 ) then
     return
  end if
  L1 = 2
  L2 = NSTEP
  if ( NROUGH>0 ) ROUGH=.TRUE.
  ITER = 0                  ! FIRST PASS FLAG
  !@@@  
  CRTRAD = FCTRAD*float(NAKA-1)
  IDELTA = 1
  !@@@  
  do k = L1, L2
     RAD = k-1
     IDR(k)=1
     if ( LOGSC .and. k>NAKA ) then
        if ( (RAD+1.0)>=CRTRAD .or. k==L2 ) then
           CRTRAD = FCTRAD*RAD
           IDR(k) = IDELTA
           IDELTA = 1
        else
           IDR(k)=0
           IDELTA = IDELTA+1
           cycle  ! go to 10
        end if
        !@@@  
     end if
  end do !10   continue
  !@@@
  do j=1,3
     do i=1,8
        eb(i,j) = 999999.0
     end do
  end do
  
  !     Find a profile solution.  Make NITER passes.  On each pass find
  !     a solution for each contour step.  At the end of the pass,
  !     examine the contour parameters, and iterate.
  
  NTABLE = 1
  NITOLD = NITER
  
1000 continue
  
  !     Enter here from interactive input ----------------------------------
  
  do KLOOP=1, NITER
     if ( ITER==1 ) then
        profs(1,1) = profs(1,2)
        profs(2,1) = profs(2,2)
        X          = profs(1,1)
        Y          = profs(2,1)
        profs(5,1) = XBIN(X,Y,gdata,jext,iext)
        V(1)       = profs(5,1)
     end if
     
     print '(/'' ITERATION  '',I3,'' /'',I3/ &
          & /'' Rmaj     Xc      Yc   C   Angle    Ecc'', &
          & 5X,''Mu    cos4E  sin4E  cos2E  sin2E'')', NTABLE, NITOLD
     
     if ( ERRLOG ) &
          & write(NLP,'(/'' ITERATION  '',I3,'' /'',I3/ &
          &      /3X,''Rmaj     Yc      Xc       Angle    Ecc'', &
          &       5X,''Mu    cos4E  sin4E  cos2E  sin2E'')') NTABLE, NITOLD
     
     ANG_old = POSANG + 90.0 + ANGSGN*profs(3,1)
     ANG_old = mod(ANG_old,180.0)
     if ( ANG_old<0.0 ) ANG_old = ANG_old + 180.0
     
     if ( profs(5,1)>0.0 ) then
        amag = -2.5*log10( profs(5,1) ) + sky
     else
        amag = 40.0
     end if
     
     print '(F5.1,1X,2F8.2,2X,F8.2,F8.3,F8.3,4F7.2)', &
          &   0.0,profs(1,1),profs(2,1), 180.0-ANG_old, &
          &   profs(4,1), amag, (profs(L,1),L=6,9)
     
     if ( ERRLOG )  &
          &  write(NLP ,'('' '',F5.1,1X,2F8.2,2X,F8.2,F8.3,F8.3,4F7.2)') &
          &        0.0,profs(1,1),profs(2,1),180.0-ANG_old, &
          &        profs(4,1), amag, (profs(L,1),L=6,9)
     
     do i=1,8
        SUMFC(i) = 0.0                 ! Sum of Fourier coefficients
     end do
     NNFCI = 0                         ! Number of inner contours
     NNFCO = 0                         ! Number of outer contours
     
     do K=L1, L2
        
        !           MAJOR AXIS LENGTH
        RAD = K-1
        !@@@
        if ( LOGSC .and. K>NAKA ) then
           if ( IDR(K)>0 ) then
              !@@@  
              ISGM = IDR(K)-1         !  = should be -2?
           else
              do L=1,12
                 profs(L,K) = 0.0
              end do
              profs(5,K) = SKPINT
              cycle ! go to 30
              !@@@
           end if
        end if
        !@@@
        
        ! A priori elements for ellipse. If ITER=0 use current values of Xc, etC
        ! If ITER=1 use a priori contour from previous pass.
        
        if ( ITER==1 ) then
           Xc  = profs(1,K)
           Yc  = profs(2,K)
           ANG = profs(3,K)
           ECC = profs(4,K)
        end if
        
        CS = cos(ANG/57.29578)
        SN = sin(ANG/57.29578)
        B  = RAD*sqrt(1.-ECC**2)
        NC = 2.*PI*RAD+1
        NC = (NC/4)*4
        if ( RAD<=2.0 ) NC=4*NC
        DE = 2.*PI/NC
        
        !     Get intensity of image along a priori contour.  Use the values
        !     of the isophote angle, eccentricity and center to find where
        !     the contour falls in the image.  Interpolate to find the
        !     intensity of image at integral steps along the contour.
        
        do L=1, NC
           E  = L*DE
           DX = RAD*cos(E)
           DY =   B*sin(E)
           X  = Xc+DX*CS-DY*SN
           Y  = Yc+DX*SN+DY*CS
           if ( K<=NSINC ) then
              !              USE 2D SINC INTERPOLATION
              PRF(L) = XBIN(X,Y,gdata,jext,iext)
              IBF    = 3
              !@@@
           else if ( LOGSC .and. K>NAKA .and. ISGM>=1 ) then
              PRF(L) = AVRBIN(X,Y,ISGM,gdata,jext,iext)
              !@@@  
              IBF    = 0
           else if ( ROUGH ) then
              !@@@              USE BILINEAR INTERPOLATION
              PRF(L) = BIN(X,Y,gdata,jext,iext)
              IBF    = 1
           else
              !@@@              USE SHORTER SINC
              PRF(L) = OUTBIN(X,Y,gdata,jext,iext)
              IBF    = 2
           end if
        end do
        
        !     Points L, NC-L, NC/2-L,NC/2+L are equivalent. Fill in points
        !     less than or equal to -100000, or lost to zones or edges.
        !     Use the average value of the positive equivalent points.
        
        INCFLG = 0          ! INCOMPLETE CONTOUR FLAG
        do L=1,NC/4
           if ( PRF(L)<=DLINE .or. PRF(NC-L)<=DLINE .or. &
                &  PRF(NC/2-L)<=DLINE .or. PRF(NC/2+L)<=DLINE ) then
              NPX = 0
              PX  = 0.0
              if ( PRF(L)>DLINE ) then
                 NPX = NPX+1
                 PX  = PX+PRF(L)
              end if
              if ( PRF(NC-L)>DLINE ) then
                 NPX = NPX+1
                 PX  = PX+PRF(NC-L)
              end if
              if ( PRF(NC/2-L)>DLINE ) then
                 NPX = NPX+1
                 PX  = PX+PRF(NC/2-L)
              end if
              if ( PRF(NC/2+L)>DLINE ) then
                 NPX = NPX+1
                 PX  = PX+PRF(NC/2+L)
              end if
              if ( NPX==0 ) then
                 INCFLG = 1 ! CONTOUR IS INCOMPLETE
              else
                 PX = PX/NPX
                 if ( PRF(L)     <=DLINE ) PRF(L)     =PX
                 if ( PRF(NC-L)  <=DLINE ) PRF(NC-L)  =PX
                 if ( PRF(NC/2-L)<=DLINE ) PRF(NC/2-L)=PX
                 if ( PRF(NC/2+L)<=DLINE ) PRF(NC/2+L)=PX
              end if
           end if
        end do
        if ( PRF(NC)  <=DLINE ) PRF(NC)=PRF(NC/2)
        if ( PRF(NC/2)<=DLINE ) PRF(NC/2)=PRF(NC)
        if ( PRF(NC)<=DLINE .and. PRF(NC/2)<=DLINE ) INCFLG=1
        if ( INCFLG==0 ) then
           NNZ = NC
        else
           !     Contour is incomplete.  Zero out complementary parts of contour.
           print '('' Following contour is incomplete:'')'
           write(NLP ,'('' FOLLOWING CONTOUR IS INCOMPLETE:'')')
           do L=1, NC/4-1
              if ( PRF(L)<=DLINE ) then
                 PRF(  NC/4-L) = DLINE
                 PRF(  NC/4+L) = DLINE
                 PRF(3*NC/4-L) = DLINE
                 PRF(3*NC/4+L) = DLINE
              end if
           end do
           
           if ( PRF(NC)<=DLINE ) then
              PRF(  NC/4) = DLINE
              PRF(3*NC/4) = DLINE
           end if
           
           if ( PRF(NC/4)<=DLINE ) then
              PRF(  NC  ) = DLINE
              PRF(  NC/2) = DLINE
              PRF(3*NC/4) = DLINE
           end if
           
           NNZ = 0
           do L=1,NC
              if ( PRF(L)>DLINE ) then
                 NNZ = NNZ+1
              else
                 PRF(L) = 0.0
              end if
           end do
        end if
        
        !     Get Fourier coefficients along contour.  The N=1 Sine AND Cosine
        !     coefficients show how accurate the contour center is.  The N=2
        !     Sine and Cosine coefficients are a measure of the accuracy of the
        !     contour position angle and eccentricity.  The N=4 Sine and Cosine
        !     coefficients are large if the contour is not elliptical.
        
        B0 =  PRF(NC/4) +PRF(  NC/2)  +PRF(3*NC/4)+PRF(NC)
        A1 = -PRF(NC/2) +PRF(  NC  )
        B1 =  PRF(NC/4) -PRF(3*NC/4)
        A2 = -PRF(NC/4) +PRF(  NC/2)  -PRF(3*NC/4)+PRF(NC)
        B2 = 0.0
        A4 = B0
        B4 = 0.0
        do L=1,NC/4-1
           E   = L*DE                       ! PIXEL ANGLE
           C1  = cos(E)
           S1  = sin(E)
           C2  = C1*C1 - S1*S1              ! COSINE(2*ANG)
           S2  = 2.*S1*C1                   ! SINE(2*ANG)
           C4  = C2*C2 - S2*S2              ! COSINE(4*ANG)
           S4  = 2.*S2*C2                   ! SINE(4*ANG)
           P1  = PRF(     L)
           P2  = PRF(NC/2-L)
           P3  = PRF(NC/2+L)
           P4  = PRF(NC  -L)
           SUM = P1 + P2 + P3 + P4
           B0  = B0 + SUM                   ! CONTOUR INTENSITY
           A1  = A1 + C1*(P1-P2-P3+P4)      ! HARMONICS:
           B1  = B1 + S1*(P1+P2-P3-P4)
           A2  = A2 + C2*SUM
           B2  = B2 + S2*(P1-P2+P3-P4)
           A4  = A4 + C4*SUM
           B4  = B4 + S4*(P1-P2+P3-P4)
        end do
        
        !     Normalize the coefficients by the number of contour points.
        
        if ( NNZ==0 ) then
           print '('' Number of points = 0 '')'
           write(NLP ,'('' NUMBER OF POINTS = 0 '')')
           NNZ=1
        end if
        B0 =    B0/NNZ
        A1 = 2.*A1/NNZ
        B1 = 2.*B1/NNZ
        A2 = 2.*A2/NNZ
        B2 = 2.*B2/NNZ
        A4 = 2.*A4/NNZ
        B4 = 2.*B4/NNZ
        
        !     If contour is incomplete COS(4E) is totally screwed up.
        
        if ( INCFLG==1 ) A4=0.0
        profs( 1,K) = Xc
        profs( 2,K) = Yc
        profs( 3,K) = ANG
        profs( 4,K) = ECC
        profs( 5,K) = B0
        profs( 6,K) = A4
        profs( 7,K) = B4
        profs( 8,K) = A2
        profs( 9,K) = B2
        profs(10,K) = A1
        profs(11,K) = B1
        
        if ( K<=16 ) then
           do I=1,4
              SUMFC(I) = SUMFC(I) + profs(I+7,K)*profs(I+7,K)
           end do
           NNFCI = NNFCI + 1
        else
           do I=5,8
              SUMFC(I) = SUMFC(I) + profs(I+3,K)*profs(I+3,K)
           end do
           NNFCO = NNFCO + 1
        end if
        
        !999
        if ( omit .and. K>=L_omit1 .and. K<=L_omit2 ) then
           profs( 1,K) = profs( 1,K-1)
           profs( 2,K) = profs( 2,K-1)
           profs( 3,K) = profs( 3,K-1)
           profs( 4,K) = profs( 4,K-1)
           profs( 5,K) = B0
           profs( 6,K) = 0.0
           profs( 7,K) = 0.0
           profs( 8,K) = 0.0
           profs( 9,K) = 0.0
           profs(10,K) = 0.0
           profs(11,K) = 0.0
           V(K)        = B0
           cycle   !  go to 30
        end if
        
        
        !     Use the Fourier coefficients to update the eccentricity, position
        !     angle, and center of the contours if this is the first pass,
        !     otherwise, wait till all contours are complete before updating
        !     parameters.
        !@@@
        if ( ITER==0 .and. K<=NDERIV ) then
           DERIV = profs(5,K) - profs(5,K-IDR(K))
           DERIV = DERIV/float(IDR(K))
           !@@@  
           ANG_old = ANG
           FMULT=1.0
           if ( K==2 ) FMULT=.3
           if ( K==3 ) FMULT=.4
           if ( K==4 ) FMULT=.5
           if ( K>=5 .and. K<=7 ) FMULT=.6
           FRAC=FMULT*FINC
           !              if ( K<=7 ) then
           !                 FRAC = 0.5
           !              else
           !                 FRAC = 1.0
           !              end if
           !
           call XYAEINC(Xc,Yc,ANG,ECC,DERIV,RAD,A1,B1,A2,B2,CENTER, &
                &       ANGFRE,RATFIX,ERRLOG,FRAC,NLP)
           
           !V             if ( K<=3 .and. ECC>0.6 ) ECC=0.6
           !ANG           if ( K<4 ) ANG = ANG_old
           
           !     The following IF-block is to keep contours from crossing.
           
           if ( K>2 .and. ( .NOT.LOGSC .or. ( LOGSC .and. K<=NAKA ) ) ) then
              RKM = 0.9*float(K)*sqrt(1.0-profs(4,K-1)**2)
              RK  = float(K+1)*sqrt(1.0-ECC*ECC)
              if ( RK<RKM ) then
                 ECC = sqrt(1.0-(RKM/float(K+1))**2)
              end if
           end if
        end if
        
        ! Push the contour intensity into the vector array
        
        V(K)    = profs(5,K)
        ANG_old = POSANG + 90.0 + ANGSGN*profs(3,K)
        ANG_old = mod(ANG_old,180.0)
        if ( ANG_old<0.0 ) ANG_old = ANG_old + 180.0
        
        if ( profs(5,K)>0.0 ) then
           amag = -2.5*log10( profs(5,K) ) + sky
        else
           amag = 40.0
        end if
        
        print '(F6.1,2F8.2,I2,F8.2,F8.3,F8.3,4F7.2,a1,$)', &
             &   (K-1)*psize,profs(1,K),profs(2,K),IBF,180.0-ANG_old, &
             &   profs(4,K),amag,(profs(L,K),L=6,9), char(13)
        
        if ( ERRLOG )  &
             &  write(NLP ,'('' '',F6.1,2F8.2,I2,F8.2,F8.3,F8.3,4F7.2)')  &
             &       (K-1)*psize,profs(1,K),profs(2,K),IBF,180.0-ANG_old, &
             &       profs(4,K),amag,(profs(L,K),L=6,9)
        
     end do !  30      continue
     
     WTFCI = float(NNFCI)
     WTFCO = float(NNFCO)
     do i=1,4
        eb(i,1) = sqrt( SUMFC(i)/WTFCI )
        eb(i,3) = min(eb(i,1),eb(i,3))
     end do
     do i=5,8
        eb(i,1) =0.0
        if ( WTFCO>0. ) eb(i,1) = sqrt( SUMFC(i)/WTFCO )
        eb(i,3) = min(eb(i,1),eb(i,3))
     end do
     
     print *
     print *
     print '(16X,''Inner 15 contours'',20X,''Outer contours''/ &
          &      '' Mean of   A2       B2       A1       B1'', &
          &      ''        A2       B2       A1       B1''/ &
          &      ''         ( Ecc.     PA       Xc       Yc'', &
          &      ''        Ecc.     PA       Xc       Yc )''/ &
          &      '' Min.:'',2(4F9.3,1X)/'' Prev:'',2(4F9.3,1X)/ &
          &      '' Curr:'',2(4F9.3,1X)/)',  ( ( eb(i,j),i=1,8 ),j=3,1,-1 )
     
     if ( ERRLOG ) &
          & write(NLP,'(''0'',15X,''Inner 15 contours'',20X, &
          &             ''outer contours''/ &
          &             '' '',''Mean of   A2       B2       A1       B1'', &
          &             ''         A2       B2       A1       B1''/ &
          &             '' '',8X,''( Ecc.     PA       Xc       Yc'',9X, &
          &             ''Ecc.     PA       Xc       Yc )''/ &
          &             '' '',''Min.:'',2(4F9.3,2X)/'' '',''Prev:'',2(4F9.3,2X)/ &
          &             '' '',''Curr:'',2(4F9.3,2X)/)') ((eb(i,j),i=1,8), j=3,1,-1)
     
     do i=1,8
        eb(i,2) = eb(i,1)
     end do
     
     !     All the contours are complete.  Take the radial intensity
     !     derivatives, median filter them, and use them to update
     !     the angles, eccentricities, and if desired, the centers
     !     of the contours.
     
     ITER   = 1
     iterate = 0
     if ( INTER .and. ( KLOOP==NITER ) ) then
        !           Interactive Control
        print '(''Interactive iteration ? (y/<n>) ===> '',$)'
        read  '(A)', YN
        if ( yn=='y' ) iterate = 1
        if ( iterate==1 ) then
           print '(      '' Enter fractional increment: 0<fi<=1 ===> '',$)'
           read *, FINC
           if ( FINC>1.0 .or. FINC<=0.0 ) then
              print *, 'Fi = 0.5 was assigned'
              Finc=0.5
           end if
           print '('' Solve for center ? (y/<n>) ===> '',$)'
           read '(A)', YN
           if ( yn=='y' ) then
              CENTER = .TRUE.
           else
              CENTER = .FALSE.
           end if
           NROUGH = 1
        end if
     end if
     
     !     If no further iterations are desired, we are finished.  correct
     !     the position angles of the contours, and load the intensities
     !     into a spectrum buffer.
     
     if ( KLOOP==NITER .and. iterate==0 ) then
        do i=1, NSTEP
           ANG_old = POSANG + 90.0 + ANGSGN*profs(3,i)
           ANG_old = mod(ANG_old,180.0)
           if ( ANG_old<0.0 ) ANG_old = ANG_old + 180.0
           profs(3,i) = ANG_old
        end do
        
        !LOUT       Save the resulting profile into the disk file
        
        write(LOUT,'(''             Surface photometry profile'')')
        write(LOUT,'(''   '',A18,T45,A2,''-band'')') TITLE, band
        write(LOUT,'(/ ''   Major axis profile'',T45,''Mean profile'')')
        write(LOUT,'(''   N'',5X,''R"'',5X,''LOG(R)'',3X, &
             &       ''Mu'',3X,''R**0.25'',T43,''  N'',5X, &
             &       ''Rm"'',3X,''LOG(Rm)'',3X,''Mu'',3X, &
             &       ''Rm**0.25'',T84,''Ellipticity   PA'',  &
             &       ''        Xc        Yc ''/)')
        do I=1, NSTEP
           RR    = (I-1)*psize
           RLOG  = -5.0
           if ( I>1 ) RLOG=log10(RR)
           R4    = RR**(0.25)
           BOA   = sqrt(1.0-profs(4,I)*profs(4,I))
           ELP   = 1.0 - BOA
           RM    = RR*sqrt(BOA)
           RLOGM = -5.0
           if ( I>1 ) RLOGM=log10(RM)
           R4M   = RM**(0.25)
           if ( V(I)>0.0 ) then
              amag = -2.5*log10( V(I) ) + sky
           else
              amag = 40.0
           end if
           
           write(LOUT,'(1X,I3,F8.2,1X,F8.3,1X,F7.3,1X,F6.3,T43,I3, &
                &       F8.2,1X,F8.3,1X,F7.3,1X,F6.3,T87,F5.3,F10.2,2f10.3)') &
                &     I-1,RR,RLOG,amag,R4,I-1,RM,RLOGM,amag,R4M,ELP, &
                &     180.0-profs(3,I),profs(1,I),profs(2,I)
        end do
        
        return
        !     Normal exit of profile subroutine
     end if
     !     =====================================================================
     if ( NDERIV>=L1 ) then
        do K=L1, NDERIV
           if ( IDR(K)>0 ) then
              DERIV = profs(5,K) - profs(5,K-IDR(K))
              DERIV = DERIV/float(IDR(K))
              profs(12,K) = DERIV
           else
              profs(12,K) = 0.0
           end if
        end do
     end if
     !@@@  
     K = MEDANG/2
     !@@@  
     if ( LOGSC ) then
        do I=6, NAKA-K
           do J=I-K,I+K
              SORT(J-I+K+1) = profs(12,J)
           end do
           call FMEDIAN(SORT,MEDANG,AMED)
           HOLD(I) = AMED
        end do
        do I=6 ,NAKA-K
           profs(12,I) = HOLD(I)
        end do
     else
        do I=6, NSTEP-K
           do J=I-K,I+K
              SORT(J-I+K+1) = profs(12,J)
           end do
           call FMEDIAN(SORT,MEDANG,AMED)
           HOLD(I) = AMED
        end do
        do I=6 ,NSTEP
           if (I>NSTEP-K) then
              profs(12,I) = HOLD(NSTEP-K)
           else
              profs(12,I) = HOLD(I)
           end if
        end do
     end if
     !@@@  
     !     ADJUST THE CONTOUR PARAMETERS
     
     do K=L1, L2
        FMULT=1.0
        if ( K==2 ) FMULT=.3
        if ( K==3 ) FMULT=.4
        if ( K==4 ) FMULT=.5
        if ( K>=5 .and. K<=7 ) FMULT=.6
        FRAC=FMULT*FINC
        !           if ( K<=7 ) then
        !              FRAC = 0.5
        !           else
        !              FRAC = FINC
        !           end if
        !@@@
        if ( profs(5,K)>SKPINT ) then
           call XYAEINC(profs(1,K),profs(2,K),profs(3,K),profs(4,K),profs(12,K), &
                &      K-1., profs(10,K),profs(11,K),profs(8,K),profs(9,K), &
                &       CENTER,ANGFRE,RATFIX,ERRLOG,FRAC,NLP)
        end if
        !@@@  
        !V             if (K<=3 .and. profs(4,K)>0.6) then
        !V                 profs(4,K) = 0.6
        !V             end if
        !
        ! The following IF-block is to keep contours from crossing.
        
        if ( K>2 .and. (.NOT.LOGSC .or. ( LOGSC .and. K<=NAKA )) ) then
           RKM = 0.9*float(K)*sqrt(1.0-profs(4,K-1)**2)
           RK  = float(K+1)*sqrt(1.0-profs(4,K)**2)
           if ( RK<RKM ) then
              profs(4,K) = sqrt(1.0-(RKM/float(K+1))**2)
           end if
        end if
     end do
     
     !HAMABE     ! AVOID ABRUPT CHANGE OF ECC. (omitted by M.Hamabe 99-09-05)
     !         do K=NSINC*2+1, NSTEP
     !            if ( IDR(K)>0 ) then
     !               Diff_Ecc = abs( profs(4,K)-profs(4,K-IDR(K)) )
     !               if ( Diff_Ecc>0.1 ) profs(4,K) = profs(4,K-IDR(K))
     !            end if
     !         end do
     !HAMABE     ! AVOID ABRUPT CHANGE OF ECC.
     
     ! Median filter angles and eccentricities of outer contours
     
     K = MEDANG/2
     IEND = NSTEP - K
     if ( LOGSC )  IEND = NAKA-K
     do I=6, IEND
        ANG_old = profs(3,I)
        do J=I-K,I+K
           if ( profs(3,J)-ANG_old>90.0 ) then
              SORT(J-I+K+1) = profs(3,J)-180.0
           else if ( profs(3,J)-ANG_old<-90.0 ) then
              SORT(J-I+K+1) = profs(3,J)+180.0
           else
              SORT(J-I+K+1) = profs(3,J)
           end if
        end do
        call FMEDIAN(SORT,MEDANG,ANGMED)
        HOLD(I) = ANGMED
     end do
     
     do I=6, IEND
        profs(3,I) = HOLD(I)
     end do
     
     K = 2
     do I=6, IEND
        do J=I-K,I+K
           SORT(J-I+K+1) = profs(4,J)
        end do
        call FMEDIAN(SORT,5,ANGMED)
        HOLD(I) = ANGMED
     end do
     
     do I=6 ,IEND
        profs(4,I) = HOLD(I)
     end do
     
     !     Use average of angles for inner contours.
     !     Modified Feb. 25, 1986 to use mean PA for r=0 to 2
     !     Instead of 0 to 3. JK
     !     Variable MPA can be set by the keyword MPA that
     !     also sets NOTFRE TO TRUE. MB
     !     Modified Mar. 12, 1986 to use mean PA for r=0 TO MPA-1.
     !     Default MPA=3. JK
     
     ANG = 0.0
     
     !     logical variable NOTFRE added
     
     if ( NOTFRE .and. NSTEP>6 ) then
        ANG_old = profs(3,3)
        MPA1    = MPA  + 1
        MPA2    = MPA1 + 3
        do K = MPA1, min(MPA2,NSTEP)
           if ( profs(3,K)-ANG_old>90.0 ) then
              ANG_old =       profs(3,K) - 180.0
              ANG     = ANG + profs(3,K) - 180.0
           else if ( ANG_old-profs(3,K)>90.0 ) then
              ANG_old =       profs(3,K) + 180.0
              ANG     = ANG + profs(3,K) + 180.0
           else
              ANG     = ANG + profs(3,K)
              ANG_old =       profs(3,K)
           end if
        end do
        
        ANG = ANG/(  min(MPA2,NSTEP) - MPA1 + 1   )
        
        do K = 1, MPA
           profs(3,K) = ANG
        end do
     end if
     
     !     If the center has been adjusted, take the center for the central
     !     4 contours as the average of the next 7 contours.
     
     !        if ( CENTER .and. L2>4 ) then
     !           SUMX = 0.0
     !           SUMY = 0.0
     !           SUMN = 0.0
     !           do K=5, min(11,L2)
     !              SUMX = SUMX+profs(1,K)
     !              SUMY = SUMY+profs(2,K)
     !              SUMN = SUMN+1.0
     !           end do
     !           SUMY = SUMY/SUMN
     !           SUMX = SUMX/SUMN
     !           do K=1,4
     !              profs(1,K) = SUMX
     !              profs(2,K) = SUMY
     !           end do
     !        end if
     
     if ( KLOOP>=NROUGH ) ROUGH=.FALSE.
     
     NTABLE = NTABLE + 1
     
  end do
  
  !     A complete pass has been completed go back and do it again.
  !     (The exit occurs in the middle of the loop)
  
  NITER = 1
  if ( NROUGH>0 ) ROUGH = .TRUE.
  go to 1000
900 print *, ' ERROR OPENING DISK FILE FOR OUTPUT'
  
999 return
end Subroutine SUBPROF
      
!_______________________________________________________________________
      
SUBROUTINE XYAEINC (Xc, Yc, ANG, ECC, DERIV, RAD, A1, B1, A2, B2, &
     &              CENTER, ANGFRE, RATFIX, ERRLOG, FINC, NLP)
  
  !     This subroutine is used to update the contours found in the
  !     profile subroutine.  It was written by Steve Kent and modified by
  !     Tod Lauer
  !     Updatede by M.Hamabe  1988-06-28
  
  parameter (PI=3.14159)
  logical    CENTER, ERRLOG
  logical    ANGFRE, RATFIX
  
  Ecc_0 = Ecc
  !                : adjust center flag.
  if ( DERIV>0.0 ) then
     !        print '('' Derivative > 0.0'')'
     if ( ERRLOG ) write(NLP,'('' Derivative > 0.0'')')
     DERIV = -DERIV
  end if
  !     Guard against a Derivative very close to zero that can cause the
  !     division to overflow.
  if ( abs(DERIV)<=1.0E-20 ) then
     !        print '('' Derivative <> 0.0'')'
     if ( ERRLOG ) write(NLP, '('' Derivative <> 0.0'')')
     return
  else
     AA1 = A1/(DERIV*RAD)
     AA2 = A2/(DERIV*RAD)
     BB1 = B1/(DERIV*RAD)
     BB2 = B2/(DERIV*RAD)
  end if
  F0 = 1./sqrt(1.-ECC**2)
  if ( AA2>0.0 .and. F0==1.0 ) then
     ANG = ANG+90.
     AA2 = -AA2
  end if
  !     Sometimes don't want to increment by entire step size.
  AA2 = FINC*AA2
  BB2 = FINC*BB2
  AA1 = FINC*AA1
  BB1 = FINC*BB1
  if ( abs(AA2)<=1.0 ) then
     if ( AA2>0.0 ) then
        F2 = F0**2/(1.+4.*AA2)
     else
        F2 = F0**2*(1.-4.*AA2)
     end if
  else
     !        print '(''    Corr. for ECC. could not be determined.'')'
     if ( ERRLOG ) write(NLP,'(''    Corr. for ECC. could not be determined.'')')
     F2=F0**2
  end if
  
  !     Revise the contour angle and eccentricity
  if ( F2<1.0 ) F2=1.0
  if ( F2>=1.0 ) ECC=sqrt(1.0-1.0/F2)
  if ( F2==1.0 ) then
     if ( BB2==0.0 ) then
        PSI = 0.0
     else if ( AA2==0.0 ) then
        PSI = PI/2.0
     else
        PSI = 0.5*ATAN(BB2/AA2)
     end if
  else
     ARG = 4.*F0*BB2/(1.-F2)
     !        ARG = AMAX1(ARG,-1.)
     !        ARG = AMIN1(ARG,1.)
     ARG = max(ARG,-.866)
     ARG = min(ARG, .866)
     !        if ( abs(ARG)<1.0 ) then
     PSI = 0.5*asin(ARG)
     !        else if
     !           print '(''    Corr. for P.A. could not be determined.'')'
     !              write(NLP,
     !    +             '(''    Corr. for P.A. could not be determined.'')')
     !              PSI =0.0
     !        end if
  end if
  ANG_old = ANG
  if ( ANGFRE ) ANG = ANG + PSI*57.29578
  DXT = RAD*AA1
  DYT = RAD/sqrt(F2)*BB1
  DX = -DXT
  DY = -DYT
  if ( DX<-1.0 ) DX=-1.0
  if ( DX> 1.0 ) DX= 1.0
  if ( DY<-1.0 ) DY=-1.0
  if ( DY> 1.0 ) DY= 1.0
  !     Revise the contour center
  CS = cos(ANG/57.29578)
  SN = sin(ANG/57.29578)
  if ( CENTER ) then
     Xc = Xc + DX*CS - DY*SN
     Yc = Yc + DX*SN + DY*CS
  end if
  ANG = mod(ANG,180.0)
  if ( ANG<=-90.0 ) ANG = ANG + 180.0
  if ( ANG > 90.0 ) ANG = ANG - 180.0
  
  if ( RATFIX ) Ecc = Ecc_0
  
  return
end SUBROUTINE XYAEINC
